package airlift.util;

/**
 * Title:        airlift.util.Enumerate
 * Project:      AirLift Framework for Java
 * Copyright:    Copyright (c) 2001 -
 *                  Note: this file is hereby released for reference use
 *                  to participants in the Middle Tennessee Java Users Group.
 * Company:      The AirLift Team
 * @author       Mark Soileau
 * @version      1.4
 */

import java.util.*;
import java.io.*;

public abstract class Enumerate
        implements Serializable
{
    private final String mName;
    private final int mIndex;
            // made mIndex non-transient for quicker identification on deserialize  - jms


    /**
     * This is the default constructor which constructs a non ordinal
     * Enumerate instance which can be used to access the enum list
     * of the concrete class through various methods.
     * This is also called by the deserialization code during object deserializaton.
     * The default constructor should not be used to construct normal ordinal
     * Enumerates.
     */
    protected Enumerate()
        {
        this(null);
        }

    /**
     * This is the constructor which adds a new Enumerate to the enum list
     * associated with the concrete class.  If requires a String which serves
     * as the "name" of the Enumerate.  The findByName() method below will
     * look through the concrete enum list in search of an Enumerate with
     * a matching name.  This is the only constructor which adds a new Enumerate to
     * the related (by class) concrete list.
     */
    protected Enumerate(String pName)
        {
        this.mName = pName;
        if (pName != null)
            {
            List enumList = this.getConcreteEnumList();
            this.mIndex = enumList.size();
            enumList.add(this);
            }
        else {
            mIndex = -1;
            } // null name get us an "access to enum list" instance.
        }

    /**
     * Through this method, each concrete extension must provides its own
     * static final List (usually java.util.ArrayList internally) for use in storage
     * of ordered list of Enumerate extensions.
     * There is a potential issue here with protected access.  Methods in the same package
     * as concrete classes currently can gain access to the "internal" ArrayList.
     * One solution is to add a static section in each concrete class which makes the
     * internal List "unmodifyable".
     */
    protected abstract List getConcreteEnumList();

    /**
     * Get the ordinal index of this Enumerate instance within its concrete set of related
     * (by concrete class) enums.
     */
    public int getIndex()
        {
        return mIndex;
        }

    /**
     * Get the String name associated with this Enumerate instance.
     */
    public String getName()
        {
        return mName;
        }

    /**
     * This returns an iterator across an unmodifyable List built from the
     * concrete enum ArrayList provided by the concrete class.  It can be
     * used to search concrete enum lists for Enumerates with specified
     * attributes.  This cannot be called as a static method so must be
     * called using an instance of the conrete Enumerate desired.  Using the
     * default (parameterless) constructor is an obvious way to acheive access
     * to the enum list without creating a new enum within the list.
     */
    public Iterator iterate()
        {
        //return Collections.unmodifiableList(this.getConcreteEnumList()).iterator();
        // we're now depending on the concrete class to make the underlying
        // enum list unmodifiable in its own static initialization code.
        return this.getConcreteEnumList().iterator();
        }

    /**
     * Get the size of the enum list
     */
    public int getSize()
        {
        return this.getConcreteEnumList().size();
        }

    /**
     * Performs a linear search of the concrete enum list searching for a matching
     * name.  If found, the related Enumerate is returned, else returns null.
     */
    public Enumerate findByName(String pName)
        {
        // TODO, support optional hashmap which can allow quicker
        // lookups by String name.. this can be controlled by having
        // concrete classes return a reference to a static HashMap
        // in an overriden method that normally returns null.

        Iterator iter = this.getConcreteEnumList().iterator();
        while (iter.hasNext())
            {
            Enumerate enum = (Enumerate)iter.next();
            if (enum.mName.equals(pName))
                {
                return enum;
                }   // if names match
            }   // iterate ordereed enum list

        return null;
        }

    /**
     * This method returns an Enumerate by its index within the concrete enum list.
     * If will throw an IndexOutOfBoundsException if there is no such Enumerate index.
     */
    public Enumerate get(int pIndex)
            throws IndexOutOfBoundsException
        {
        return (Enumerate)this.getConcreteEnumList().get(pIndex);
        }

    /**
     * This method is provided to insure a canonical object deserialization such
     * that we do not end up with multiple instances of the same Enumerate.
     */
    private Object readResolve()
            throws ObjectStreamException
        {
        return this.getConcreteEnumList().get(this.mIndex);
        }

    /**
     * Overrides base class method convenience; used in XML generation, etc.
     */
    public String toString()
        {
        return this.mName;
        }

}
